"{ Package: 'jv:libgdbs/tests' }"

TestCase subclass:#GDBDebuggerTests
	instanceVariableNames:'debugger simulator'
	classVariableNames:''
	poolDictionaries:''
	category:'GDB-Core-Tests'
!

!GDBDebuggerTests class methodsFor:'documentation'!

documentation
"
    documentation to be added.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!GDBDebuggerTests methodsFor:'running'!

tearDown
    simulator notNil ifTrue:[ simulator stop ].
    debugger notNil ifTrue:[ debugger release ].

    "Created: / 06-09-2014 / 02:17:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerTests methodsFor:'tests - basic'!

test_basic_01
    | timeouted |

    debugger := GDBDebugger new.
    self assert: debugger isConnected.
    timeouted  := ([
        debugger send: (GDBMI_gdb_exit new).
        true.
    ] valueWithTimeout: 3 seconds) isNil.

    self assert: timeouted not.
    self assert: debugger isConnected not.

    "Created: / 24-06-2014 / 09:06:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 06-09-2014 / 02:16:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_basic_02
    simulator := GDBSimulatorProcess new record: GDBSimulatorResource session_factorial_01.
    simulator start.
    debugger := GDBDebugger newWithProcess: simulator.

    "Created: / 24-06-2014 / 09:09:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-09-2014 / 02:18:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerTests methodsFor:'tests - threads'!

test_inferiors_01a
    | tgevent tevent |

    simulator := GDBSimulatorProcess new record: GDBSimulatorResource session_factorial_01.
    simulator start.
    debugger := GDBDebugger newWithProcess: simulator.
    debugger announcer when: GDBThreadGroupEvent do:[:ev |  
        tgevent := ev.
    ].
    debugger announcer when: GDBThreadEvent do:[:ev |  
        tevent := ev.
    ].

    debugger send: (GDBMI_file_exec_and_symbols new arguments: {'/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial'}).
    debugger send: (GDBCLICommand new value: 'b factorial').

    self assert: debugger inferiors size == 1.
    self assert: debugger inferiors anElement id = 'i1'.
    "/ No, following won't work as thread-group-added event is
    "/ emited before we register the handler above!!
    "/ self assert: debugger inferiors anElement == event threadGroup.

    debugger send: (GDBCLICommand new value: 'r').
    Delay waitForMilliseconds: 100. "/ Give it chance to catch up with events.  

    self assert: tgevent notNil.
    self assert: tevent notNil.
    self assert: debugger inferiors size == 1.
    self assert: debugger inferiors anElement pid = 7719.
    self assert: debugger inferiors anElement == tgevent threadGroup.
    self assert: debugger inferiors anElement threads size == 1.
    self assert: debugger inferiors anElement threads anElement == tevent thread.
    self assert: debugger inferiors anElement threads anElement isTerminated not.
    self assert: debugger inferiors anElement threads anElement isRunning not.
    self assert: debugger inferiors anElement threads anElement isStopped.
    self assert: debugger inferiors anElement threads anElement stack size == 2. 

    tgevent := tevent := nil.

    debugger send: (GDBCLICommand new value: 'del 1').
    debugger send: (GDBCLICommand new value: 'c').
    Delay waitForMilliseconds: 100. "/ Give it chance to catch up with events.  

    self assert: tgevent notNil.
    self assert: debugger inferiors size == 1.
    self assert: debugger inferiors anElement exitCode = 23.
    self assert: debugger inferiors anElement == tgevent threadGroup.
    self assert: debugger inferiors anElement threads size == 1.
    self assert: debugger inferiors anElement threads anElement == tevent thread.
    self assert: debugger inferiors anElement threads anElement isTerminated .
    self assert: debugger inferiors anElement threads anElement isRunning not.
    self assert: debugger inferiors anElement threads anElement isStopped not.

    "Created: / 07-09-2014 / 13:37:11 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 16-09-2014 / 23:47:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBDebuggerTests class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

